// Newton Game Dynamics 1.0 (NGD)  
// Game Studio plugin interface to NGD: by Julio Jerez 08/05/2003
////////////////////////////////////////////////////////////////////////
define USE_NEWTON_GAME_DYNAMICS;


//*******************************************************************************************************
// Global Variables an constants
//*******************************************************************************************************
// main handle to Newton Game Dynamics 1.0 (NGD)
var_nsave newtonHandle;


// unit system converstion factors
define QUANT_IS_INCH, 0.025;
define QUANT_IS_FEET, 0.305;
define QUANT_IS_METER, 1.00;
define QUANT_IS_CENTIMETER, 0.01;

// collsion primitive types
define USE_SPH_COLLISION, 0;
define USE_BOX_COLLISION, 1;

//*******************************************************************************************************
// System functions
//*******************************************************************************************************
// NewtonStart:
// description: Initilize the engine
dllfunction NewtonStart(logo);

// NewtonExit:
// description: destroy the engine and all physics entities
dllfunction NewtonExit();

// NewtonUpdate:
// description: advance the simulation by one time step
dllfunction NewtonUpdate (timestep);


// NewtonFreeze:
// description: stop simulation from advancing
dllfunction NewtonFreeze ();

// NewtonFreeze:
// description: tell simumlation to continue updates
dllfunction NewtonUnfreeze ();


// NewtonSetMinimumFPS:
// description: Set the minimu frames per seconds for the simulation. The default FPS is 60 fps
//	            fps should be a value between 30 and 600.			
// remarks: When the frame rate of the simulation is lower than the set value the engine will
//          preform substeps to meet the minimum fps.
//			for example if you set fps to 60 and the frame rate of the game is 60.0 or higher then the engine
//			will take one time step per call. but if the fps become 59 then the engine will take two time step.
dllfunction NewtonSetMinimumFPS (fps);


// NewtonSetFrictionTreshold:
// description: Set the minimun value that a normal force contact must be in ordr to considered 
//              for friction calculation. the default value is 0.05
// remarks: in general the collision phase find more contact than there are needed to prevent 
//			interpenetration. when this happens some of the values of normal forces are very low
//			and they are no part of the set needed for friction calculation.
//          Ideally they should be zero but there aren't, so they become pollution that the friction
//          solver need to eliminate at a enormous cost in calculation. This values can be identified 
//			and rejected early by checking that the value of the mass of the body multiply by the 
//			accelThreshold is lower than normal force.
//          This translate to significant gain in performance with negelgible quality lost, specially 
//			for very large piles of bodies
dllfunction NewtonSetFrictionTreshold (accelThreshold);


// NewtonSetFrictionDegradationCount
// description: Set the maximun number of normal forces that the solve will consider for friction calculation.
//              Count should be a number between 50 and 2700. The default value is 400. 
// remarks: When the number of normal force for a pile of bodies is larger than the specified count, these forces
//          are sorted by there acceleration influence in the friction, (normal force magnitud multiply by the 
//			mass of the body). then the force with lower influence are remove arbitrarialy until the maximun limit
//			is reached.	The next effect is that that contact become frictionless.
//          This is a safety guard measure that prevent slowdown on large piles a the spence of quality degradation
//			is the solver.         
dllfunction NewtonSetFrictionDegradationCount (count);




// NewtonRemoveAllBodies:
// description: destroy all physics entities, including the map (usefull for changing levels)
dllfunction NewtonRemoveAllBodies ();

// NewtonRemoveAllBodies :
// description: set the scale factor to convert from Game Studio Quant to Meters
// remark: the default scale value is one QUANT represent one INCH
dllfunction NewtonSetUnitSystem (scale);

// NewtonSetBodyLeaveWorldEvent :
// description: set the function event for when a rigid body leave the world
// remark: the function event have the form NewtonBodyLeaveWorldEvent (body)
//		   The function NewtonAddMap set the objectLeaveWorld to zero
dllfunction NewtonSetBodyLeaveWorldEvent (NewtonBodyLeaveWorldEvent);

// NewtonSetWorlSize :
// description: set the sizeof the newtor world
// remark: use this function to set the maximun size of the world
//		   if the NewtonBodyLeaveWorldEvent is set and a rigid body crosses the boundary of this box. 
//		   the rigid body is frozen, and the then event NewtonBodyLeaveWorldEvent is called to notify the
//		   the script that this body had left the world.	
//		   The function NewtonAddMap set the World side to the default values
//		   min(-10000, -10000, -10000), max(10000, 10000, 10000)
dllfunction NewtonSetWorlSize (&min, &max);

//*******************************************************************************************************
// Physics entities creation funtions
//*******************************************************************************************************
// NewtonAddMap:
// description: add background map to NGD 
// parameters: 
//		mapName is the name of the map file (level,wmp)
//		splashScreen: conitec splash screen
// return: one if sussesful
// remarks: 
//		if splashScreen is set to the conitec splashpannel, the engine will wait until the panel is set off
//		then it will display the newton logo for 3 second.
//		if splashScreen is zero the the newton logo will be show immideatly 
dllfunction NewtonAddMap (mapMame, splashScreen);


// NewtonRemoveMap:
// description: destroy the level map
// parameters: 
// return: 
// remarks: detsroy the map
dllfunction NewtonRemoveMap(); 


// NewtonAddEntity:
// Description: add a map entity to NGD 
// Parameters: 
//    entity is the game studio entity
//    collision geometry type 
// Return: A handle to the Physics Body represetative of the entity, zero if the entity was already found 
//		   in NGD 	
// remarks: this function can only be call for entities created with the editor
dllfunction NewtonAddMapEntity (entity, collisionType);


// NewtonAddEntity:
// Description: add a terrain entity to NGD 
// Parameters: 
//    entity is the game studio entity
//    collision geometry type 
// Return: A handle to the Physics Body represetative of the entity, zero if the entity was already found 
//		   in NGD 	
// remarks: this function can only be call for entities that are position with the editor
dllfunction NewtonAddTerrainEntity (entity);



// NewtonSpawnEntity:
// Description: add a entity to NGD 
// Parameters: 
//    entity is the game studio entity
//    collision geometry type 
// Return: A handle to the Physics Body represetative of the entity, zero if the entity was already found 
//		   in NGD 	
// remarks: this function can be called to create Map entites and Runtime entities. 
//			but this function can not take compound collision
dllfunction NewtonSpawnEntity (entity, collisionType);


// NewtonRemoveEntity 
// Description: remove and entity from NGD 
// Parameters: entity is the game studio entity
// Return: one
dllfunction NewtonRemoveEntity (entity);


//*******************************************************************************************************
// Physics material creation and interaction funstion
//*******************************************************************************************************
// NewtonCreateMaterial: 
// Description: create a physics material
// Parameters: 
// Return: handle to material id
// remark: When a new material id is created all posible Material Interaction with pre-exiting
//	       materials are set to default values. 
dllfunction NewtonCreateMaterial ();


// NewtonMaterialSetCollidable: 
// Description: enable or disable collission between these material by default
// Parameters: 
// Return: handle to material id
// remark: 
dllfunction NewtonMaterialSetCollidable (mat0, mat1, state);

// NewtonMaterialSetFriction: 
// Description: Set the Static and kinetic coeficient of friction between to materials
// Parameters: .....
// Return: 1
// remark: The friction values will be clamped between 0.0 and 1.0
//		   The static coef. of friction value must be greather than the kenitic coef. of friction.
//		   If the static coef. of friction is set to zero then the material become frictionLess.
dllfunction NewtonMaterialSetFriction (mat0, mat1, staticFriction, kineticFriction);


// NewtonMaterialSetElasticity: 
// Description: set the elacticity (how fast and contact point will bounce back
// Parameters: .......
// Return: 1
// remark: The elastic values will be clamped between 0.1 and 0.9
dllfunction NewtonMaterialSetElasticity (mat0, mat1, elasticCoef);


// NewtonMaterialSetDepthRecover: 
// Description: set the Penetration coef (how fast interpenetration of two body is resolved)
// Parameters: mat0 mat1 are materials, value is coeficient of friction
// Return: 1
// remark: the penetration coef. will be clamped between 0.1 and 0.9
dllfunction NewtonMaterialSetDepthRecover (mat0, mat1, value);


// NewtonMaterialSetSoundEffect: 
// Description: set sound to be played when Mat0 and Mat1 collides 
// Parameters: 
// Return: 1
// remark: these function store the sound with the material pair Mat0 and Mat1
//		   When the collision event function in called from the newton DLL, the Script Even can query this sound 
//		   the material pair.  		
dllfunction NewtonMaterialSetSoundEffect (mat0, mat1, sound);



// NewtonMaterialCollisionBegin: 
// Description: Set the collsion event handler functions.
// Parameters: 
// Return: 1
// remark: when to entities collide a series of conctact is generated. if the script desired to get 
//		   feedback information form the collision, then it must pass the adress of the event handles function 
//		eventCollisionBegin (materialPair);
//		eventCollisionContact (materialPair, contact);
//		eventCollisionEnd (materialPair);
dllfunction NewtonMaterialCollisionBegin (mat0, mat1, eventCollisionBegin);
dllfunction NewtonMaterialCollisionContact (mat0, mat1, eventCollisionContact);
dllfunction NewtonMaterialSetCollisionEnd (mat0, mat1, eventCollisionEnd);


//*******************************************************************************************************
// 
// the following functions can only be call from inside a NewtonMaterialCollisionContact event.
// All cantact parameters are set to the defulat values asigned to the material pair at creation time. 
// This make the calling of the floowing functions optional. but they can be call to make produce play
// effects.
// 
//*******************************************************************************************************

// NewtonMaterialDisable: 
// Description: disable this contact
// Parameters: 
// Return: 
// remark: if this function is called from a NewtonMaterialCollisionContact the contact will be rejected
dllfunction NewtonMaterialDisable (matPair);

// NewtonMaterialSetContactRestitution: 
// Description: 
// Parameters: 
// Return: 
// remark: 
dllfunction NewtonMaterialSetRestitution (matPair, restitution);

// NewtonMaterialSettSoftness: 
// Description: 
// Parameters: 
// Return: 
// remark: 
dllfunction NewtonMaterialSetSoftness (matPair, softness);

// NewtonMaterialRotateContactFriction: 
// Description: 
// Parameters: 
// Return: 
// remark: 
dllfunction NewtonMaterialRotateFriction (matPair, &directionVector);


// NewtonMaterialSetFrictionState: 
// Description: 
// Parameters: 
//			   matPair:  handle to matA - matB permutation
//             contact: contact handle
//             state: 1 enable friction, 0 disable friction
//             direction: 0 apply to friction direction0, 1 apply to friction direction 1 
// Return: 
// remark: 
dllfunction NewtonMaterialSetFrictionState (matPair, state, index);


// NewtonMaterialSetStaticFrictionCoef: 
// Description: 
// Parameters: 
//			   material material handle
//             contact: contact handle
//             state: 1 enable friction, 0 disable friction
//             direction: 0 apply to friction direction0, 1 apply to friction direction 1 
// Return: 
// remark: 
dllfunction NewtonMaterialSetStaticFrictionCoef (matPair, coef, index);

// NewtonMaterialSetKineticFrictionCoef: 
// Description: 
// Parameters: 
//			   material material handle
//             contact: contact handle
//             state: 1 enable friction, 0 disable friction
//             direction: 0 apply to friction direction0, 1 apply to friction direction 1 
// Return: 
// remark: 
dllfunction NewtonMaterialSetKineticFrictionCoef (matPair, coef, index);

// NewtonMaterialGetNormalSpeed: 
// Description: calculate the impact speed of the contact point 
// Parameters: 
// Return: speed
// remark: this function calculate the speed of the contact along the contact normal
dllfunction NewtonMaterialGetNormalSpeed (matPair, contact);

//this function was deprecated use NewtonMaterialGetNormalSpeed (material, contact) instead
//dllfunction NewtonMaterialGetContactSpeed (matPair, contact);

// NewtonMaterialGetTangentSpeed: 
// Description: calculate tangent speed of the contact
// Parameters: 
// Return: speed
// remark: this function calculate the speed of the contact along the tangent to the contact normal 
dllfunction NewtonMaterialGetTangentSpeed (matPair, contact, index);

// NewtonMaterialSetTangentAcceleration: 
// Description: calculate tangent speed of the contact
// Parameters: 
// Return: speed
// remark: this function calculate the speed of the contact along the tangent to the contact normal 
dllfunction NewtonMaterialSetTangentAcceleration (matPair, accelMag, index);


// NewtonMaterialGetSoundEffect: 
// Description: Query the sound pointer from the material pair matPair
// Parameters: 
// Return: 1
// remark: This function is usually called from and collision event handler 
dllfunction NewtonMaterialGetSoundEffect (matPair);


// NewtonMaterialUserId: 
// Description: Query the sound pointer from the material pair matPair
// Parameters: 
// Return: 1
// remark: This function is usually called from and collision event handler 
dllfunction NewtonMaterialGetUserId (matPair);




//*******************************************************************************************************
//
// Physics entities manipulation functions
//
//*******************************************************************************************************
// NewtonGetBody: 
// Description: given and entity return the body assosiated with it
// Parameters: 
// return body or zero if there is no body associated with this entity: 
dllfunction NewtonGetBody (entity);

// NewtonEntityFormCollision: 
// Description: Get the entityof a compound collision entity
// Parameters: 
// return body or zero if there is no body associated with this entity: 
dllfunction NewtonEntityFormCollision(entity);

// NewtonGetBodyMass: 
// Description: Get the mass of a physic body
// Parameters: mass value. 
// return the body mass: 
dllfunction NewtonGetBodyMass (body);

// NewtonSetBodyMass: 
// Description: Set the mass of a physic body
// Parameters: mass value. 
// remark: a zero or negative mass value make the object static.
dllfunction NewtonSetBodyMass (body, mass);

// NewtonSetBodyActiveState: 
// Description: insert or remove body from active simulation array
// Parameters:  ....
// remark: !!!!!!! important that you understand this !!!!!!!!!
//		   If TrueFalse argument is set to one, the body will alway be activated.
//		   however if TrueFalse is zero body will be frozen. 
//		   The Newton Game Dynamics make an extreme use of frame to frame coherance. If you try to 	   	
//         take over autoactivation by calling NewtonSetBodyActiveState every frame at run time, 
//		   it will result in severe performance penalties.	 
dllfunction NewtonSetBodyActiveState (body, TrueFalse);

// NewtonGetBodyActiveState: 
// Description: insert or remove body from active simulation array
// Parameters:  ....
// remark: !!!!!!! important that you understand this !!!!!!!!!
//		   If TrueFalse argument is set to one, the body will alway be activated.
//		   however if TrueFalse is zero body will be frozen. 
//		   The Newton Game Dynamics make an extreme use of frame to frame coherance. If you try to 	   	
//         take over autoactivation by calling NewtonSetBodyActiveState every frame at run time, 
//		   it will result in severe performance penalties.	 
dllfunction NewtonGetBodyActiveState (body);


// NewtonSetBodyAutoActiveState: 
// Description: determine wether inclusion or exclusion of a body into the active simulation array
//				is control by the Newton Engine or by the Script.	
// Parameters:  ....
// remark: When the parameter TrueFalse is one, NGD will take care of activation and deactivation of  
//         bodies automatically. This is the default value of a body at creation time. It is also the 
//         recomended mode for the vast majority of bodies. However some bodies need to set auto 
//         activation off. For example if the player is a Physics entity, or there are AI controlled 
//		   Physic entities, etc. If this the case. then you may set auto activation off for that body 
//		   and call NewtonSetBodyActiveState (body, 1) in order to keep the body active.
// 		   Setting Auto activation to zero and calling NewtonSetBodyActiveState (body, 0) make no sense, 
//		   and will result in unpredictable random behavior.
//		   This function have no penalty, but bera in mind that keeping too many active bodies will 
//		   decrease performance.
dllfunction NewtonSetBodyAutoActiveState (body, TrueFalse);


// NewtonSetBodyAutoActiveCutOff: 
// Description: Render a body deactive when the Magnitud of the Velocity and Acceleration
//				is below the Cuoff value
// Parameters:  ....
// remark: The default and minimun allowable CutOff value is 0.1 
//		   The auto activation algorithm in NGD is quite robust, however,
//		   because realtime physics is executed at discrete time stepping, it is
//		   possible that a body reach a Dynamic Equelibrium condition. 
//		   This is when a body rock back and forward between two differebt states, 
//		   the acceleration is usually bellow the cutoff value but the speed keep oscilation 
//		   around zero but never falling bellow the cutoff value. You may have to experiment 
//		   by increasing these cutoff values should this situation happend in your game
dllfunction NewtonSetBodyAutoActiveCutOff (body, speedValue, accelValue);

// NewtonSetTransform: 
// Description: Reset the tranformation matrix of the	body
// Parameters:  .....
// remark: When it is nessesary to teleport an entiry to a different location, after
//         setting the new position and orientaion angles on the entity, this function 
//         will reset the transformation matrix of the body.
//		   This function should be use with care. Calling this function on a linked	body
//		   without properly calling it for the others bodies will case umpredictable result. 	 
//		   This function do not have any effect if from called inside a NewtonUpdate step. 	
//		   When calling this function for a body make sure that the new destination position
//		   is free of collision. 	
dllfunction NewtonBodySetTransform (body);


// NewtonBodyGetVelocity: 
// Description: Get the body velocity
// Parameters:  .....
// remark: 
dllfunction NewtonBodyGetVelocity (body, &velocity);

// NewtonBodySetVelocity: 
// Description: Set the body velocity
// Parameters:  .....
// remark: 
dllfunction NewtonBodySetVelocity (body, &velocity);


// NewtonBodyGetOmega: 
// Description: Get the body angular velocity
// Parameters:  .....
// remark: 
dllfunction NewtonBodyGetOmega (body, &omega);

// NewtonBodySetOmega: 
// Description: Set the body angular velocity
// Parameters:  .....
// remark: 
dllfunction NewtonBodySetOmega (body, &omega);




// NewtonBodyAddForce: 
// Description: set body force
// Parameters:  .....
// remark: 
dllfunction NewtonBodyAddForce (body, force_x, force_y, force_z);

// NewtonBodyAddTorque: 
// Description: set body torque
// Parameters:  .....
// remark: 
dllfunction NewtonBodyAddTorque (body, torque_x, torque_y, torque_z);

// NewtonBodyAddBuoyancy: 
// Description: add the buoyancy force to a body immerse in water
// Parameters:  fluidData liquid parameters
//				fluidData[0..3] = fluid suface plane
//				fluidData[4]    = fluid density
//				fluidData[5]    = fluid linear viscuisity coef
//				fluidData[6]    = fluid angular viscuisity coef
//				fluidData[7]    = gravity
//				fluidData[8]    = body densinty
// remark: 
dllfunction NewtonBodyAddBuoyancy (body, &fluidData);

// NewtonGetBuoyancyPlane: 
// Description: return the surface plane of the water entity. this the upper surfafe
//              of the bounding box of the water entity				
//Parameters:  
// remark: 
dllfunction NewtonGetBuoyancyPlane (waterEntity, &plane);


// NewtonGetPointVelocity
// Description: calculate the velocity of a rigid body a point 
// Parameters: 
//		pointVelocity rigid body (in global space)  
// return: the magnitud of the velocity at a point
// remark: the argument pointVelocity is overwritten with the value
//		   of the vector velocity at that point	
//dllfunction NewtonGetPointVelocity (body, &pointVelocity);

// NewtonBodyAddImpulse: 
// Description: apply an impulse to a rigig body
// Parameters:  
//		body: body
//		impulse: record
//			record[0...2] = impact point on target
//			record[3...5] = copy origin of impulse (use to determine the direction of the impulse)
//			record[6] = mass of hitting body
//          record[7] = desired change of speed apply to hit body
// remark: repeated applycation of impulse to the same body may increase the velocity
//		   of the body beyond the limits of stability	
dllfunction NewtonBodyAddImpulse (body, &record);





// NewtonSetBodyMass: 
// Description: asign a material of a physic body
// Parameters:  
// remark: When a bady is created the default material is automatically assigned to it.
dllfunction NewtonSetBodyMaterial (body, material);

// NewtonSetBodyLinearDamp: 
// Description: set speed resistance
// Parameters:  .....
// remark: the linear resistance use the Viscuos Resistance model the represent resistance.
//		   That is, the resistance is proportional to the square of the speed of the entity	
dllfunction NewtonSetBodyLinearDamp (body, coef);


// NewtonSetBodyAngularDamp: 
// Description: set speed resistance
// Parameters:  .....
// remark: the angular resistance use the Viscuos Resistance model
//		   that is, the resistance is proportional to the square of the angular speed of the entity	
dllfunction NewtonSetBodyAngularDamp (body, coef_x, coef_y, coef_z);

// NewtonSetBodyForceAndTorque: 
// Description: pass the address of a function pointer in charge to apply external force
//				and external torque to the body.
// Parameters:  ....
// remark: !!!!!!! important that you understand this !!!!!!!!!
//			the format of the function pointer is  "function EventHandler(body)" 
//		    external force and external torque can be modify from within a event callback. 
//			any attempt to do atherwise will result in
//			an unpredictable random behavior.
//          This event function is only called when a body is active. 
dllfunction NewtonSetBodyForceAndTorque (body, eventHandler);



//*******************************************************************************************************
// Scene graph utility
//*******************************************************************************************************


// NewtonGetChildEntity, NewtonGetParentEntity, NewtonGetSiblingEntity
// Description: get the adress of parent and children entites set in the editor
//				
// Parameters:  ....
// remark: allow to navegate the scenegraph setup by the editor.
//		   this function are usefull to setup constraints visually
dllfunction NewtonGetChildEntity (parentEntity);
dllfunction NewtonGetNextChildEntity (childEntity);
dllfunction NewtonGetParentEntity (childEntity);



//*******************************************************************************************************
// Joints Utilities
//*******************************************************************************************************
// return body attached to the joint
dllfunction NewtonJointGetEntity0 (joint); 
dllfunction NewtonJointGetEntity1 (joint); 

// destroy joint 
dllfunction NewtonDestroyJoint (joint); 


//*******************************************************************************************************
// Ball constraint 
//*******************************************************************************************************
// NewtonCreateBallConstraint
// Description:  create a ball socket constraint
// Parameters:
//			pivotEntity: auxiliary entity use to connect to rigid bodies
//			attachBody: child body (ex: this will be the door of a building)
//			refBody: reference body (ex: this will be the door frame) 
//			controller: function call back use to controll the hinge behaviour (this parameter could be zero for free hinges)
dllfunction NewtonCreateBallConstraint (pivotEntity, refBody, spinBody, controller); 


//*******************************************************************************************************
// Hinge constraint 
//*******************************************************************************************************
// NewtonCreateHingeConstraint
// Description:  create a hinge constraint
// Parameters:
//		pivotEntity: auxiliary entity use to connect to rigid bodies
//		attachBody: child body (ex: this will be the door of a building)
//		refBody: reference body (ex: this will be the door frame) 
//		controller: function call back use to controll the hinge behaviour (this parameter could be zero for free hinges)
dllfunction NewtonCreateHingeConstraint (pivotEntity, attachBody, refBody, controller); 

// return hinge angular speed in degree per secunds
dllfunction NewtonGetHingeOmega (hinge); 

// return hinge angle in degree
dllfunction NewtonGetHingeAngle (hinge);



//*******************************************************************************************************
// Dry friction constraint
//*******************************************************************************************************
// NewtonAddDryFriction
// Description: add dry friction to simgle spherical body when they are free rolling
// Parameters:	ballbody
//				friction coeficient
// remark: this is a very special constraint that should be applied only to spherical objects.
//		   this contracint is inactive if any of the following contions fail
//			- the body has a spherical collision, 	 
//			- the body in in contact with another body
//          - the body is not contraint to any other body
//          - the body is rolling over the surcafe of another body
//          if all these conditions are meet then a friction force in apply at the center of the body
//          the force is proportinal to the weight of the body and oposite the tangent linear velocity of the body
//          You should not use this constraint on body that don't need it
dllfunction NewtonAddDryFriction (ballBody, frictionCoef); 



//*******************************************************************************************************
//*******************************************************************************************************
// 
//	Vehicle utitity interface 
// 
//*******************************************************************************************************
//*******************************************************************************************************

// NewtonCreateVehicleConstraint: 
// Description: create and attach a vehicle constraint to a body
// Parameters: 
//				vehicle = body for this vehicle
//				controlBegin = event function of the form controlBegin (vehicleBody);
//				controlTyre = event function of the form controlTyre (vehicleBody, tyreIndex);
//				controlEnd = event function of the form controlEnd (vehicleBody);
// return: the handle to the vehicle contraint
// remark: 
dllfunction NewtonCreateVehicleConstraint (vehicleBody, controlBegin, controlTyre, controlEnd);


// NewtonVehicleAddTyre: 
// Description: create and attach a vehicle constraint to a body
// Parameters: vehicleConst 
//	tyreEntity.skill1; // tyre mass 
//	tyreEntity.skill2; // max steer angle 
//	tyreEntity.skill3; // suspension span 
//	tyreEntity.skill4; // suspension shock
//	tyreEntity.skill5; // suspension Stiffness
//	tyreEntity.skill6; // minimun friction
// return: the tyre index 
// remark: this function try to add a tyre to the vehicle. after all the tyres are added you must call
//	       the function NewtonVehicleBalance ()
dllfunction NewtonVehicleAddTyre (vehicleConst, tyreEntity);

// NewtonVehicleBalanceTyres: 
// Description: calculate vehicle equielibrium point after all tyres are asigned to the vehicle
// Parameters: vehicleConst,
//			   weight of the vehicle	 
// remark: this function must be called after all the tyres are added to the vehicle
dllfunction NewtonVehicleBalanceTyres (vehicleConst, weight);





// NewtonGetTyreSteerParam: 
// Description: return the parametric steer angle for this tyre
// Parameters: vehicleConst
//			   tyreIndex
// remark: 
dllfunction NewtonGetTyreOmega (vehicleConst, tyreIndex);
dllfunction NewtonGetTyreRadius (vehicleConst, tyreIndex);
dllfunction NewtonGetTyreInertia (vehicleConst, tyreIndex);
dllfunction NewtonGetTyreSteerParam (vehicleConst, tyreIndex);
dllfunction NewtonGetTyreNormalLoad (vehicleConst, tyreIndex);
dllfunction NewtonGetTyreLateralSpeed (vehicleConst, tyreIndex);
dllfunction NewtonGetTyreLongitudinalSpeed (vehicleConst, tyreIndex);


// NewtonGetTyreSteerParam: 
// Description: return the parametric steer angle for this tyre
// Parameters: vehicleConst
//			   tyreIndex
//			   paramtricAngle
// remark: 
dllfunction NewtonSetTyreSteerParam (vehicleConst, tyreIndex, paramtricAngle);
dllfunction NewtonSetTyreLateralFrictionCoef (vehicleConst, tyreIndex, frictionCoef);
dllfunction NewtonSetTyreLongituFrictionCoef (vehicleConst, tyreIndex, frictionCoef);
dllfunction NewtonSetTyreTorque (vehicleConst, tyreIndex, torque);



//*******************************************************************************************************
//*******************************************************************************************************
//
// Physics Main loop and termination
//
//*******************************************************************************************************
//*******************************************************************************************************

bmap newtonBimap = <newtonsplash.tga>; 
panel NewtonLogo 
{
	bmap = newtonBimap;
	flags = refresh,d3d,transparent;
	layer = 0;
}

function NewtonShowLogo ()
{
	var t;
	t = 0;	

	// freeze the simulation
	NewtonFreeze ();
	
	// wait for the game to start
	wait (10);
	while (freeze_mode == 1) {
		wait (1);
	}

	
	// stop the action
	freeze_mode = 1;
	NewtonLogo.pos_x = (screen_size.x - bmap_width(newtonBimap))/2;
	NewtonLogo.pos_y = (screen_size.y - bmap_height(newtonBimap))/2;
	NewtonLogo.visible = on;
	
	// fade in logo
	while (t < 100) {
		t = t + 1;
		NewtonLogo.alpha = t;
		wait (1);
	}
	
	// fade out logo
	while (t) {
		t = t - 1;
		NewtonLogo.alpha = t;
		wait (1);
	}
	
	// release resuorces
	NewtonLogo.visible = off;
	bmap_purge(newtonBimap);
	
	// wait another 10 secounds
	wait (10);
	
	// continue the action
	freeze_mode = 0;
		
	// continue simulaion	
	NewtonUnfreeze ();
}

// newton physics main loop
string newtonDllName = <newton.dll>;
starter NewtonMain() 
{
	newtonHandle = dll_open (newtonDllName);
	NewtonStart(NewtonLogo);
	
	// wait for the system to be initialized
	wait(3);

	
	// show the logo
	NewtonShowLogo ();
	
	// call the initial setup
	NewtonSetupSystem ();

	// set the friction limits
	NewtonSetFrictionTreshold (1.0);
	NewtonSetFrictionDegradationCount (300);
	
	// run forever
	while (1) { 
		// update physics every frame
		dll_handle = newtonHandle;
		NewtonUpdate (time);
		wait(1);
	}
}

// destroy physics on exits
function game_exit()
{
	dll_handle = newtonHandle;
	NewtonExit();
	dll_close (newtonHandle);
	EXIT;
}

// this will do the minimal setup
function NewtonSetupSystem() 
{
	// set unit system	
	dll_handle = newtonHandle;
	NewtonSetUnitSystem (QUANT_IS_INCH);
	
	// set material interaction propeteries (0, 0)
	NewtonMaterialSetFriction (0, 0, 1.0, 0.8);
	NewtonMaterialSetElasticity (0, 0, 0.5);
	NewtonMaterialSetDepthRecover (0, 0, 0.2);
}